from unittest import TestCase

# Brazil import
from ..america import (
    Brazil, BrazilAcre, BrazilAlagoas, BrazilAmapa, BrazilAmazonas,
    BrazilBahia, BrazilCeara, BrazilDistritoFederal, BrazilEspiritoSanto,
    BrazilGoias, BrazilMaranhao, BrazilMatoGrosso, BrazilMatoGrossoDoSul,
    BrazilPara, BrazilParaiba, BrazilPernambuco, BrazilPiaui,
    BrazilRioDeJaneiro, BrazilRioGrandeDoNorte, BrazilRioGrandeDoSul,
    BrazilRondonia, BrazilRoraima, BrazilSantaCatarina, BrazilSaoPauloState,
    BrazilSergipe, BrazilTocantins,
)
# Canada territories
from ..america import (
    Canada,
    Ontario,
    Quebec,
    BritishColumbia,
    Alberta,
    Saskatchewan,
    Manitoba,
    NewBrunswick,
    NovaScotia,
    PrinceEdwardIsland,
    Newfoundland,
    Yukon,
    NorthwestTerritories,
    Nunavut,

)
# Other american countries
from ..america import (
    Argentina, Chile, Colombia, Mexico, Panama, Paraguay
)

from ..registry import registry


class RegistryAmerica(TestCase):

    def _check_brazil_states(self, classes):
        self.assertIn(BrazilAcre, classes)
        self.assertIn(BrazilAlagoas, classes)
        self.assertIn(BrazilAmapa, classes)
        self.assertIn(BrazilAmazonas, classes)
        self.assertIn(BrazilBahia, classes)
        self.assertIn(BrazilCeara, classes)
        self.assertIn(BrazilDistritoFederal, classes)
        self.assertIn(BrazilEspiritoSanto, classes)
        self.assertIn(BrazilGoias, classes)
        self.assertIn(BrazilMaranhao, classes)
        self.assertIn(BrazilMatoGrosso, classes)
        self.assertIn(BrazilMatoGrossoDoSul, classes)
        self.assertIn(BrazilPara, classes)
        self.assertIn(BrazilParaiba, classes)
        self.assertIn(BrazilPernambuco, classes)
        self.assertIn(BrazilPiaui, classes)
        self.assertIn(BrazilRioDeJaneiro, classes)
        self.assertIn(BrazilRioGrandeDoNorte, classes)
        self.assertIn(BrazilRioGrandeDoSul, classes)
        self.assertIn(BrazilRondonia, classes)
        self.assertIn(BrazilRoraima, classes)
        self.assertIn(BrazilSantaCatarina, classes)
        self.assertIn(BrazilSaoPauloState, classes)
        self.assertIn(BrazilSergipe, classes)
        self.assertIn(BrazilTocantins, classes)

    def _check_canada_provinces(self, classes):
        self.assertIn(Ontario, classes)
        self.assertIn(Quebec, classes)
        self.assertIn(BritishColumbia, classes)
        self.assertIn(Alberta, classes)
        self.assertIn(Saskatchewan, classes)
        self.assertIn(Manitoba, classes)
        self.assertIn(NewBrunswick, classes)
        self.assertIn(NovaScotia, classes)
        self.assertIn(PrinceEdwardIsland, classes)
        self.assertIn(Newfoundland, classes)
        self.assertIn(Yukon, classes)
        self.assertIn(NorthwestTerritories, classes)
        self.assertIn(Nunavut, classes)

    def test_america(self):
        classes = set(registry.region_registry.values())
        self.assertIn(Brazil, classes)
        self._check_brazil_states(classes)

        self.assertIn(Canada, classes)
        self._check_canada_provinces(classes)

        self.assertIn(Argentina, classes)
        self.assertIn(Chile, classes)
        self.assertIn(Colombia, classes)
        self.assertIn(Mexico, classes)
        self.assertIn(Panama, classes)
        self.assertIn(Paraguay, classes)

    def test_brazil_subregion(self):
        classes = set(registry.get_subregions('BR').values())
        self._check_brazil_states(classes)

    def test_canada_subregion(self):
        classes = set(registry.get_subregions('CA').values())
        self._check_canada_provinces(classes)



from ..core import OrthodoxCalendar, SeriesShiftMixin
from ..registry_tools import iso_register


@iso_register('RS')
class Serbia(SeriesShiftMixin, OrthodoxCalendar):
    'Serbia'

    FIXED_HOLIDAYS = OrthodoxCalendar.FIXED_HOLIDAYS + (
        (1, 2, "Day After New Year"),
        (2, 15, "Statehood Day"),
        (2, 16, "Statehood Day"),
        (5, 2, "Labour Day Holiday"),
        (11, 11, "Armistice Day"),
    )

    # Civil holidays
    include_labour_day = True

    # Christian holidays
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_christmas = False



from datetime import date

from dateutil import relativedelta as rd

from ..core import Holiday
from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('GB')
class UnitedKingdom(WesternCalendar):
    'United Kingdom'

    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_boxing_day = True
    shift_new_years_day = True
    non_computable_holiday_dict = {
        1973: [(date(1973, 11, 14), "Royal wedding"), ],
        1977: [(date(1977, 6, 7), "Queen’s Silver Jubilee"), ],
        1981: [(date(1981, 7, 29), "Royal wedding"), ],
        1999: [(date(1999, 12, 31), "New Year's Eve"), ],
        2002: [(date(2002, 6, 3), "Queen’s Golden Jubilee"), ],
        2011: [(date(2011, 4, 29), "Royal Wedding"), ],
        2012: [(date(2012, 6, 5), "Queen’s Diamond Jubilee"), ],
        2022: [(date(2022, 6, 3), "Queen’s Platinum Jubilee bank holiday"), ],
    }

    def get_early_may_bank_holiday(self, year):
        """
        Return Early May bank holiday
        """
        day = date(year, 5, 1) + rd.relativedelta(weekday=rd.MO(1))
        desc = "Early May Bank Holiday"
        indication = "1st Monday in May"

        # Special case in 2020, for the 75th anniversary of the end of WWII.
        if year == 2020:
            day = date(year, 5, 8)
            desc += " (VE day)"
            indication = "VE day"
        return Holiday(day, desc, indication=indication)

    def get_spring_bank_holiday(self, year):
        day = date(year, 5, 30) + rd.relativedelta(weekday=rd.MO(-1))
        if year in (2012, 2002):
            day = date(year, 6, 4)
        if year in (1977,):
            day = date(year, 6, 6)
        if year == 2022:
            day = date(year, 6, 2)
        return Holiday(
            day,
            "Spring Bank Holiday",
            indication="Last Monday in May",
        ),

    def get_late_summer_bank_holiday(self, year):
        return Holiday(
            date(year, 8, 31) + rd.relativedelta(weekday=rd.MO(-1)),
            "Late Summer Bank Holiday",
            indication="Last Monday in August",
        )

    def non_computable_holiday(self, year):
        non_computable = self.non_computable_holiday_dict.get(year, None)
        return non_computable

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append(self.get_early_may_bank_holiday(year))
        days.append(self.get_spring_bank_holiday(year))
        days.append(self.get_late_summer_bank_holiday(year))
        non_computable = self.non_computable_holiday(year)
        if non_computable:
            days.extend(non_computable)
        return days


@iso_register('GB-NIR')
class UnitedKingdomNorthernIreland(UnitedKingdom):
    'Northern Ireland'

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        # St Patrick's day
        st_patrick = date(year, 3, 17)
        days.append((st_patrick, "Saint Patrick's Day"))
        if st_patrick.weekday() in self.get_weekend_days():
            days.append((
                self.find_following_working_day(st_patrick),
                "Saint Patrick substitute"))

        # Battle of boyne
        battle_of_boyne = date(year, 7, 12)
        days.append((battle_of_boyne, "Battle of the Boyne"))
        if battle_of_boyne.weekday() in self.get_weekend_days():
            days.append((
                self.find_following_working_day(battle_of_boyne),
                "Battle of the Boyne substitute"))
        return days



from ..registry_tools import iso_register
from .core import UnitedStates


@iso_register('US-SC')
class SouthCarolina(UnitedStates):
    """South Carolina"""
    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (5, 10, "Confederate Memorial Day"),
    )
    include_thanksgiving_friday = True
    include_christmas_eve = True
    include_boxing_day = True
    include_columbus_day = False



from ..core import FRI
from ..registry_tools import iso_register

from .core import UnitedStates


@iso_register('US-HI')
class Hawaii(UnitedStates):
    """Hawaii"""
    include_good_friday = True
    include_columbus_day = False
    include_election_day_even = True

    FIXED_HOLIDAYS = UnitedStates.FIXED_HOLIDAYS + (
        (3, 26, "Prince Jonah Kuhio Kalanianaole Day"),
        (6, 11, "King Kamehameha Day"),
    )

    def get_statehood_day(self, year):
        """
        Statehood Day: 3rd Friday in August.
        """
        return (
            self.get_nth_weekday_in_month(year, 8, FRI, 3),
            "Statehood Day"
        )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append(self.get_statehood_day(year))
        return days



from pyluach.dates import GregorianDate, HebrewDate

from ..core import Calendar, FRI, SAT
from ..registry_tools import iso_register


@iso_register("IL")
class Israel(Calendar):
    "Israel"
    include_new_years_day = False
    WEEKEND_DAYS = (SAT, FRI)

    def get_variable_days(self, year):
        days = super().get_variable_days(year)

        hebrew_date = GregorianDate(year=year, month=1, day=1).to_heb()
        jewish_year = hebrew_date.year

        holidays_hebrew_dates = [
            (HebrewDate(jewish_year, 6, 29), "Rosh Hashana Eve"),
            (HebrewDate(jewish_year + 1, 7, 1), "Rosh Hashana"),
            (HebrewDate(jewish_year + 1, 7, 2), "Rosh Hashana"),
            (HebrewDate(jewish_year + 1, 7, 9), "Yom Kippur Eve"),
            (HebrewDate(jewish_year + 1, 7, 10), "Yom Kippur"),
            (HebrewDate(jewish_year + 1, 7, 14), "Sukkot Eve"),
            (HebrewDate(jewish_year + 1, 7, 15), "Sukkot"),
            (HebrewDate(jewish_year + 1, 7, 21), "Shmini Atzeres Eve"),
            (HebrewDate(jewish_year + 1, 7, 22), "Shmini Atzeres"),
            (HebrewDate(jewish_year, 1, 14), "Pesach Eve"),
            (HebrewDate(jewish_year, 1, 15), "Pesach"),
            (HebrewDate(jewish_year, 1, 20), "7th of Pesach Eve"),
            (HebrewDate(jewish_year, 1, 21), "7th of Pesach"),
            (HebrewDate(jewish_year, 3, 5), "Shavout Eve"),
            (HebrewDate(jewish_year, 3, 6), "Shavout"),
        ]
        holidays_hebrew_dates += self.get_hebrew_independence_day(jewish_year)

        for holiday_hebrew_date, holiday_name in holidays_hebrew_dates:
            days.append((holiday_hebrew_date.to_pydate(), holiday_name))
        return days

    def get_hebrew_independence_day(self, jewish_year):
        """
        Returns the independence day eve and independence day dates
        according to the given hebrew year

        :param jewish_year: the specific hebrew year for calculating
                            the independence day dates
        :return: independence day dates
                 in the type of List[Tuple[HebrewDate, str]]
        """
        month = 2
        day = 5
        original_hebrew_independence_date = HebrewDate(jewish_year, month, day)
        if original_hebrew_independence_date.weekday() == 6:
            day = 4
        if original_hebrew_independence_date.weekday() == 7:
            day = 3
        if original_hebrew_independence_date.weekday() == 2:
            day = 6
        return [
            (HebrewDate(jewish_year, month, day - 1), "Independence Day Eve"),
            (HebrewDate(jewish_year, month, day), "Independence Day")
        ]



from datetime import timedelta
from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('DK')
class Denmark(WesternCalendar):
    'Denmark'

    include_holy_thursday = True
    include_good_friday = True
    include_easter_sunday = True
    include_easter_monday = True
    include_ascension = True
    include_whit_sunday = True
    whit_sunday_label = "Pentecost Sunday"
    include_whit_monday = True
    whit_monday_label = "Pentecost Monday"
    include_boxing_day = True
    boxing_day_label = "Second Day of Christmas"
    include_christmas_eve = True

    def get_store_bededag(self, year):  # 'great prayer day'
        easter_sunday = self.get_easter_sunday(year)
        return easter_sunday + timedelta(days=26)

    def get_variable_days(self, year):
        days = super().get_variable_days(year)
        days.append((self.get_store_bededag(year), "Store Bededag"))
        return days



from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('ES')
class Spain(WesternCalendar):
    'Spain'

    # Christian holidays
    include_epiphany = True
    include_good_friday = True
    include_assumption = True
    include_all_saints = True
    include_immaculate_conception = True

    # Civil holidays
    include_labour_day = True
    labour_day_label = "Día del trabajador"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (10, 12, "Fiesta nacional de España"),
        (12, 6, "Día de la Constitución Española")
    )


@iso_register('ES-AN')
class Andalusia(Spain):
    "Andalusia"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (2, 28, "Andalusian National Day"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-AR')
class Aragon(Spain):
    "Aragon"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (4, 23, "Aragonese National Day"),
        (12, 20, "Aragon Ombudsman Day"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-CL')
class CastileAndLeon(Spain):
    "Castile and León"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (4, 23, "Día de Castilla y León"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-CM')
class CastillaLaMancha(Spain):
    "Castilla-La Mancha"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (5, 31, "Día de la Región Castilla-La Mancha"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-CN')
class CanaryIslands(Spain):
    "Canary Islands"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (5, 30, "Día de Canarias"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-CT')
class Catalonia(Spain):
    "Catalonia"

    include_easter_monday = True
    include_boxing_day = True
    boxing_day_label = "Sant Esteve"

    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (9, 11, "Diada nacional de Catalunya"),
        (6, 24, "La revetlla de Sant Joan, Nit de Sant Joan"),
    )


@iso_register('ES-EX')
class Extremadura(Spain):
    "Extremadura"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (9, 8, "Día de Extremadura"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-GA')
class Galicia(Spain):
    "Galicia"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (5, 17, "Día das Letras Galegas"),
        (7, 25, "Santiago Apóstol o Día da Patria Galega"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-IB')
class BalearicIslands(Spain):
    "Balearic Islands"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (3, 1, "Dia de les Illes Balears"),
    )
    # Christian holidays
    include_holy_thursday = True  # Also called Maundy thursday
    include_easter_monday = True


@iso_register('ES-RI')
class LaRioja(Spain):
    "La Rioja"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (6, 9, "Dia de La Rioja"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-MD')
class CommunityofMadrid(Spain):
    "Community of Madrid"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (5, 2, "Fiesta de la Comunidad de Madrid"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-MC')
class Murcia(Spain):
    "Region of Murcia"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (6, 9, "Día de la Región de Murcia"),
        (3, 19, "San José"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-NA')
class Navarre(Spain):
    "Navarre"
    # Christian holidays
    include_holy_thursday = True  # Also called Maundy thursday
    include_easter_monday = True


@iso_register('ES-AS')
class Asturias(Spain):
    "Asturias"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (9, 8, "Día de Asturias"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-PV')
class BasqueCountry(Spain):
    "Basque Country"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (10, 25, "Euskadi Eguna"),
    )
    # Christian holidays
    include_holy_thursday = True  # Also called Maundy thursday
    include_easter_monday = True


@iso_register('ES-CB')
class Cantabria(Spain):
    "Cantabria"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (9, 15, "Día de Cantabria o Día de La Montaña"),
    )
    # Christian holiday
    include_holy_thursday = True  # Also called Maundy thursday


@iso_register('ES-VC')
class ValencianCommunity(Spain):
    "Valencian Community"
    FIXED_HOLIDAYS = Spain.FIXED_HOLIDAYS + (
        (3, 19, "San José"),
        (10, 9, "Dia de la Comunitat Valenciana"),
    )
    # Christian holiday
    include_easter_monday = True



from datetime import date
from . import GenericCalendarTest
from ..africa import (
    Algeria,
    Angola,
    Benin,
    IvoryCoast,
    Kenya,
    Madagascar,
    Nigeria,
    SaoTomeAndPrincipe,
    SouthAfrica,
)
from ..core import MON, FRI
from ..exceptions import CalendarError


class AlgeriaTest(GenericCalendarTest):
    cal_class = Algeria

    def test_year_2013(self):
        holidays = self.cal.holidays_set(2013)
        self.assertIn(date(2013, 1, 1), holidays)  # New year
        self.assertIn(date(2013, 1, 24), holidays)   # Milad un Nabi
        self.assertIn(date(2013, 5, 1), holidays)  # Labour day
        self.assertIn(date(2013, 7, 5), holidays)  # Independence day
        self.assertIn(date(2013, 8, 8), holidays)  # Eid ul-fitr
        self.assertIn(date(2013, 10, 15), holidays)  # Eid el-ada
        self.assertIn(date(2013, 11, 1), holidays)  # Anniversary revolution
        self.assertIn(date(2013, 11, 5), holidays)  # New Year
        self.assertIn(date(2013, 11, 14), holidays)  # Ashura


class BeninTest(GenericCalendarTest):
    cal_class = Benin

    def test_year_2013(self):
        holidays = self.cal.holidays_set(2013)
        self.assertIn(date(2013, 1, 1), holidays)  # new year
        self.assertIn(date(2013, 4, 1), holidays)  # easter monday
        self.assertIn(date(2013, 5, 9), holidays)   # Ascension
        self.assertIn(date(2013, 5, 20), holidays)   # Whit Monday
        self.assertIn(date(2013, 5, 1), holidays)  # Labour day
        self.assertIn(date(2013, 8, 1), holidays)  # Independence Day
        self.assertIn(date(2013, 8, 15), holidays)  # Assumption
        self.assertIn(date(2013, 10, 26), holidays)  # Armed Forces Day
        self.assertIn(date(2013, 11, 1), holidays)  # All Saints Day
        self.assertIn(date(2013, 11, 30), holidays)  # National Day
        self.assertIn(date(2013, 12, 25), holidays)  # christmas
        # Variable Muslim days
        self.assertIn(date(2013, 1, 24), holidays)  # Milad un Nabi
        self.assertIn(date(2013, 10, 15), holidays)  # Tabaski
        self.assertIn(date(2013, 8, 8), holidays)  # Eid al-Fitr


class SouthAfricaTest(GenericCalendarTest):
    cal_class = SouthAfrica

    def test_before_1910(self):
        with self.assertRaises(CalendarError):
            self.cal.holidays_set(1909)

    def test_year_2013(self):
        holidays = self.cal.holidays_set(2013)

        self.assertIn(date(2013, 1, 1), holidays)  # new year
        self.assertIn(date(2013, 3, 21), holidays)  # human rights day
        self.assertIn(date(2013, 3, 29), holidays)  # good friday
        self.assertIn(date(2013, 4, 1), holidays)  # Easter monday / Family day
        self.assertIn(date(2013, 4, 27), holidays)  # freedom day
        self.assertIn(date(2013, 5, 1), holidays)  # labour day
        self.assertIn(date(2013, 6, 16), holidays)  # youth day
        self.assertNotIn(date(2013, 6, 17), holidays)  # youth day - observed
        self.assertIn(date(2013, 8, 9), holidays)  # national women's day
        self.assertIn(date(2013, 9, 24), holidays)  # heritage day
        self.assertIn(date(2013, 12, 16), holidays)  # day of reconciliation
        self.assertIn(date(2013, 12, 25), holidays)  # christmas
        self.assertIn(date(2013, 12, 26), holidays)  # day of goodwill

        # test that Youth Day is observed on 17-Jun and is not a working day
        observed = set(map(self.cal.get_observed_date, holidays))
        self.assertIn(date(2013, 6, 17), observed)
        self.assertFalse(self.cal.is_working_day(date(2013, 6, 17)))

    def test_year_2014(self):
        # test shifting
        holidays = self.cal.holidays_set(2014)
        self.assertIn(date(2014, 4, 27), holidays)  # freedom day
        self.assertNotIn(date(2014, 4, 28), holidays)  # freedom day - observ

        # test that Freedom Day is observed on 28-Apr and is not a working day
        observed = set(map(self.cal.get_observed_date, holidays))
        self.assertIn(date(2014, 4, 28), observed)
        self.assertFalse(self.cal.is_working_day(date(2014, 4, 28)))

    def test_easter_monday(self):
        # 1910-1979, Easter Monday label was "Easter Monday"
        holidays = self.cal.holidays(1979)
        easter_monday_1979 = date(1979, 4, 16)
        holidays_dates = [item[0] for item in holidays]
        self.assertEqual(holidays_dates.count(easter_monday_1979), 1, holidays)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[easter_monday_1979], "Easter Monday")

        # From 1980 to "now", it's Family Day
        holidays = self.cal.holidays(1980)
        # Since the Founders' day is a Sunday, the same date appears twice
        # The second one is a substitute
        holidays = [item for item in holidays
                    if item[1] != "Founder's Day substitute"]
        easter_monday_1980 = date(1980, 4, 7)
        holidays_dates = [item[0] for item in holidays]
        self.assertEqual(holidays_dates.count(easter_monday_1980), 1, holidays)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[easter_monday_1980], "Family Day")

    def test_april_6th(self):
        # 1951, no April 6th
        holidays = self.cal.holidays(1951)
        holidays_dates = [item[0] for item in holidays]
        april_6th = date(1951, 4, 6)
        self.assertNotIn(april_6th, holidays_dates, holidays)

        # 1952-1973, it was named Van Riebeeck's day
        holidays = self.cal.holidays(1952)
        holidays_dates = [item[0] for item in holidays]
        april_6th = date(1952, 4, 6)
        self.assertIn(april_6th, holidays_dates, holidays)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[april_6th], "Van Riebeeck's Day")

        # 1974: No April 6th
        holidays = self.cal.holidays(1974)
        holidays_dates = [item[0] for item in holidays]
        april_6th = date(1974, 4, 6)
        self.assertNotIn(april_6th, holidays_dates, holidays)

        # 1980-1994, it became Founder's day
        holidays = self.cal.holidays(1980)
        holidays_dates = [item[0] for item in holidays]
        april_6th = date(1980, 4, 6)
        self.assertIn(april_6th, holidays_dates, holidays)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[april_6th], "Founder's Day")

        # 1995, no April 6th
        holidays = self.cal.holidays(1995)
        holidays_dates = [item[0] for item in holidays]
        april_6th = date(1995, 4, 6)
        self.assertNotIn(april_6th, holidays_dates, holidays)

    def test_workers_day(self):
        # No Workers' day before 1987
        holidays = self.cal.holidays(1986)
        holidays_labels = [item[1] for item in holidays]
        self.assertNotIn("Workers' Day", holidays_labels, holidays)

        # 1987-1989: 1st Friday in May
        holidays = self.cal.holidays(1987)
        holidays_dates = [item[0] for item in holidays]
        first_friday_may = self.cal.get_nth_weekday_in_month(1987, 5, FRI)
        self.assertIn(first_friday_may, holidays_dates)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[first_friday_may], "Workers' Day")

        # 1990-1994: No Workers' day
        holidays = self.cal.holidays(1990)
        holidays_labels = [item[1] for item in holidays]
        self.assertNotIn("Workers' Day", holidays_labels, holidays)

        # as of 1995: Workers' day is on May 1st
        holidays = self.cal.holidays(1995)
        holidays_dates = [item[0] for item in holidays]
        first_may = date(1995, 5, 1)
        self.assertIn(first_may, holidays_dates)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[first_may], "Workers' Day")

    def test_ascension_day(self):
        # Before 1993 (included), Ascension day was a holiday
        holidays = self.cal.holidays(1993)
        holidays_dates = [item[0] for item in holidays]
        ascension_1993 = self.cal.get_ascension_thursday(1993)
        self.assertIn(ascension_1993, holidays_dates)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[ascension_1993], "Ascension Day")

        # After 1993, no more Ascension day
        holidays = self.cal.holidays(1994)
        holidays_dates = [item[0] for item in holidays]
        ascension_1994 = self.cal.get_ascension_thursday(1994)
        self.assertNotIn(ascension_1994, holidays_dates)

    def test_victoria_empire_day(self):
        # 1910–1951: 24 May is Victoria Day / Empire Day
        holidays = self.cal.holidays(1951)
        holidays_dates = [item[0] for item in holidays]
        may_24th_1951 = date(1951, 5, 24)
        self.assertIn(may_24th_1951, holidays_dates, holidays)
        holidays_dict = dict(holidays)
        self.assertEqual(
            holidays_dict[may_24th_1951], "Victoria Day / Empire Day")

        # 1952-present, no Victoria / Empire day
        holidays = self.cal.holidays(1952)
        holidays_dates = [item[0] for item in holidays]
        may_24th_1952 = date(1952, 5, 24)
        self.assertNotIn(may_24th_1952, holidays_dates, holidays)

    def test_may_31st(self):
        # 1910-1960, it's Union Day
        holidays = self.cal.holidays(1960)
        holidays_dates = [item[0] for item in holidays]
        may_31st_1960 = date(1960, 5, 31)
        self.assertIn(may_31st_1960, holidays_dates, holidays)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[may_31st_1960], "Union Day")

        # 1961-1993, it's Republic Day
        holidays = self.cal.holidays(1993)
        holidays_dates = [item[0] for item in holidays]
        may_31st_1993 = date(1993, 5, 31)
        self.assertIn(may_31st_1993, holidays_dates, holidays)
        holidays_dict = dict(holidays)
        self.assertEqual(holidays_dict[may_31st_1993], "Republic Day")

        # As of 1994, no more may 31st
        holidays = self.cal.holidays(1994)
        holidays_dates = [item[0] for item in holidays]
        may_31st_1994 = date(1994, 5, 31)
        self.assertNotIn(may_31st_1994, holidays_dates, holidays)

    def test_queens_birthday(self):
        # Before 1952, no Queens Birthday
        holidays = self.cal.holidays(1951)
        holidays_dates = [item[0] for item in holidays]
        july_2nd_monday = self.cal.get_nth_weekday_in_month(1951, 7, MON, 2)
        self.assertNotIn(july_2nd_monday, holidays_dates)

        # 1952–1960: 2nd Monday in July is Queen's Birthday
        for year in (1952, 1959, 1960):  # interval of the years
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            july_2nd_monday = self.cal.get_nth_weekday_in_month(
                year, 7, MON, 2)
            self.assertIn(july_2nd_monday, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(
                holidays_dict[july_2nd_monday],
                "Queen's Birthday"
            )

        # After 1961, no more Queen's Birthday
        for year in (1961, 1962, 2008, 2018):  # interval of the years
            holidays = self.cal.holidays(year)
            holidays_labels = [item[1] for item in holidays]
            self.assertNotIn("Queen's Birthday", holidays_labels)

    def test_womens_day_label(self):
        holidays = self.cal.holidays(2018)
        womens_day = date(2018, 8, 9)
        holidays_dict = dict(holidays)
        self.assertEqual(
            holidays_dict[womens_day],
            "National Women's Day"
        )

    def test_family_day_in_july(self):
        # Before 1961, no Family day in July
        holidays = self.cal.holidays(1960)
        holidays_dates = [item[0] for item in holidays]
        family_day = date(1960, 7, 10)
        self.assertNotIn(family_day, holidays_dates)

        # From 1961 to 1973, Family day is on July 10th
        for year in (1961, 1970, 1973):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            family_day = date(year, 7, 10)
            self.assertIn(family_day, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(holidays_dict[family_day], "Family Day")

        # As of 1974, no more
        for year in (1974, 1980, 1990, 2018):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            family_day = date(year, 7, 10)
            self.assertNotIn(family_day, holidays_dates)

    def test_king_birthday(self):
        # From 1910 to 1951, 1st Monday in August
        for year in (1910, 1935, 1951):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            king_birthday = self.cal.get_nth_weekday_in_month(year, 8, MON)
            self.assertIn(king_birthday, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(holidays_dict[king_birthday], "King's Birthday")

        # Not after 1952
        for year in (1952, 1960, 1990, 2018):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            king_birthday = self.cal.get_nth_weekday_in_month(year, 8, MON)
            self.assertNotIn(king_birthday, holidays_dates)

    def test_settlers_day(self):
        # Before 1952, no settler's day
        holidays = self.cal.holidays(1951)
        holidays_dates = [item[0] for item in holidays]
        settlers_day = self.cal.get_nth_weekday_in_month(1951, 9, MON)
        self.assertNotIn(settlers_day, holidays_dates)

        # From 1952 to 1979, 1st Monday in August
        for year in (1952, 1960, 1952, 1979):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            settlers_day = self.cal.get_nth_weekday_in_month(year, 9, MON)
            self.assertIn(settlers_day, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(holidays_dict[settlers_day], "Settlers' Day")

        # Not after 1979
        for year in (1980, 1990, 2018):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            settlers_day = self.cal.get_nth_weekday_in_month(year, 9, MON)
            self.assertNotIn(settlers_day, holidays_dates)

    def test_kruger_day(self):
        # Not before 1952
        holidays = self.cal.holidays(1951)
        holidays_dates = [item[0] for item in holidays]
        kruger_day = date(1951, 10, 10)
        self.assertNotIn(kruger_day, holidays_dates)

        # Kruger Day was on October 10th 1952-1993
        for year in (1952, 1960, 1952, 1979, 1993):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            kruger_day = date(year, 10, 10)
            self.assertIn(kruger_day, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(holidays_dict[kruger_day], "Kruger Day")

        # As of 1994, not anymore
        for year in (1994, 2000, 2018):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            kruger_day = date(year, 10, 10)
            self.assertNotIn(kruger_day, holidays_dates)

    def test_december_16th(self):
        # from 1910 to 1951, it's "Dingaan's Day"
        for year in (1910, 1930, 1951):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            december_16th = date(year, 12, 16)
            self.assertIn(december_16th, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(holidays_dict[december_16th], "Dingaan's Day")

        # from 1952 to 1979 it's the "Day of the Covenant"
        for year in (1952, 1960, 1979):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            december_16th = date(year, 12, 16)
            self.assertIn(december_16th, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(
                holidays_dict[december_16th], "Day of the Covenant")

        # from 1980 to 1994 it's the "Day of the Vow"
        # NOTE: wikipedia states it starts at year 1979, but that would mean
        # there were two labels on the same year, which would be wrong.
        # We may wait to see if this error is fixed one day.
        for year in (1980, 1990, 1994):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            december_16th = date(year, 12, 16)
            self.assertIn(december_16th, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(
                holidays_dict[december_16th], "Day of the Vow")

        # As of 1995, it's the "Day of Reconciliation"
        for year in (1995, 2000, 2010, 2018):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            december_16th = date(year, 12, 16)
            self.assertIn(december_16th, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(
                holidays_dict[december_16th], "Day of Reconciliation")

    def test_december_26th(self):
        # from 1910 to 1979, it's "Boxing Day"
        # Year 1910 is excluded, since Christmas was on a Sunday, and shift
        # rules apply
        for year in (1911, 1930, 1951, 1979):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            december_26th = date(year, 12, 26)
            self.assertIn(december_26th, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(holidays_dict[december_26th], "Boxing Day", year)

        # As of 1980, it's "Day of Goodwill"
        for year in (1980, 1990, 1995, 2000, 2018):
            holidays = self.cal.holidays(year)
            holidays_dates = [item[0] for item in holidays]
            december_26th = date(year, 12, 26)
            self.assertIn(december_26th, holidays_dates)
            holidays_dict = dict(holidays)
            self.assertEqual(holidays_dict[december_26th], "Day of Goodwill")

    def test_special_1999(self):
        # National and provincial government elections – 2 June 1999
        holidays = self.cal.holidays_set(1999)
        self.assertIn(date(1999, 6, 2), holidays)
        # December 31st was a "Y2K" holiday
        self.assertIn(date(1999, 12, 31), holidays)

    def test_special_2000(self):
        holidays = self.cal.holidays_set(2000)
        # Holiday added to celebrate Y2K
        self.assertIn(date(2000, 1, 2), holidays)
        # Shift day because Jan 2nd was a sunday
        self.assertIn(date(2000, 1, 3), holidays)

    def test_special_2001(self):
        holidays = self.cal.holidays_set(2001)
        # Holiday added to celebrate Y2K
        self.assertIn(date(2001, 1, 2), holidays)

    def test_special_2004(self):
        # National and provincial government elections – 14 April 2004
        holidays = self.cal.holidays_set(2004)
        self.assertIn(date(2004, 4, 14), holidays)

    def test_special_2006(self):
        # Local government elections – 1 March 2006
        holidays = self.cal.holidays_set(2006)
        self.assertIn(date(2006, 3, 1), holidays)

    def test_special_2008(self):
        # 2 May 2008 was declared a public holiday when Human Rights Day
        # and Good Friday coincided on 21 March 2008.
        holidays = self.cal.holidays_set(2008)
        self.assertIn(date(2008, 5, 2), holidays)

    def test_special_2009(self):
        # National and provincial government elections – 22 April 2009
        holidays = self.cal.holidays_set(2009)
        self.assertIn(date(2009, 4, 22), holidays)

    def test_special_2011(self):
        # Local government elections – 18 May 2011
        holidays = self.cal.holidays_set(2011)
        self.assertIn(date(2011, 5, 18), holidays)
        # 27 December 2011 was declared a holiday by president Motlanthe
        self.assertIn(date(2011, 12, 27), holidays)

    def test_special_2014(self):
        # National and provincial government elections – 7 May 2014
        holidays = self.cal.holidays_set(2014)
        self.assertIn(date(2014, 5, 7), holidays)

    def test_special_2016(self):
        # Local government elections – 3 August 2016
        holidays = self.cal.holidays_set(2016)
        self.assertIn(date(2016, 8, 3), holidays)

    def test_special_2019(self):
        holidays = self.cal.holidays_set(2019)
        self.assertIn(date(2019, 5, 8), holidays)  # 2019 National Elections

        # Only in 2019
        holidays = self.cal.holidays_set(2017)
        self.assertNotIn(date(2017, 5, 8), holidays)


class MadagascarTest(GenericCalendarTest):
    cal_class = Madagascar

    def test_year_2013(self):
        holidays = self.cal.holidays_set(2013)
        self.assertIn(date(2013, 1, 1), holidays)  # new year
        self.assertIn(date(2013, 3, 29), holidays)  # martyrs day
        self.assertIn(date(2013, 4, 1), holidays)  # easter monday
        self.assertIn(date(2013, 5, 1), holidays)  # labour day
        self.assertIn(date(2013, 5, 9), holidays)  # ascension
        self.assertIn(date(2013, 5, 20), holidays)  # whit monday
        self.assertIn(date(2013, 6, 26), holidays)  # independence day
        self.assertIn(date(2013, 8, 15), holidays)  # assumption
        self.assertIn(date(2013, 11, 1), holidays)  # all saints
        self.assertIn(date(2013, 12, 25), holidays)  # XMas


class IvoryCoastTest(GenericCalendarTest):
    cal_class = IvoryCoast

    def test_year_2013(self):
        holidays = self.cal.holidays_set(2013)
        self.assertIn(date(2013, 1, 1), holidays)  # new year
        self.assertIn(date(2013, 4, 1), holidays)  # easter monday
        self.assertIn(date(2013, 5, 1), holidays)  # labour day
        self.assertIn(date(2013, 5, 9), holidays)  # ascension
        self.assertIn(date(2013, 5, 20), holidays)  # whit monday
        self.assertIn(date(2013, 8, 7), holidays)  # Independence day
        self.assertIn(date(2013, 8, 15), holidays)  # Assumption
        self.assertIn(date(2013, 11, 1), holidays)  # All saints
        self.assertIn(date(2013, 11, 15), holidays)  # National peace day
        self.assertIn(date(2013, 12, 25), holidays)  # XMas
        # Muslim days
        self.assertIn(date(2013, 1, 25), holidays)  # Mawlid al-Nabi
        # Laylat al-Qadr is not computable
        # self.assertIn(date(2013, 8, 3), holidays)
        self.assertIn(date(2013, 8, 8), holidays)  # End of ramadan
        self.assertIn(date(2013, 10, 15), holidays)  # Feast of sacrifice


class SaoTomeAndPrincipeTest(GenericCalendarTest):
    cal_class = SaoTomeAndPrincipe

    def test_year_2013(self):
        holidays = self.cal.holidays_set(2013)
        self.assertIn(date(2013, 1, 1), holidays)  # new year
        self.assertIn(date(2013, 2, 3), holidays)  # Martyrs' day
        self.assertIn(date(2013, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2013, 7, 12), holidays)  # Independence Day
        self.assertIn(date(2013, 9, 6), holidays)  # Armed Forces Day
        self.assertIn(date(2013, 9, 30), holidays)  # Agricultural Reform Day
        self.assertIn(date(2013, 11, 1), holidays)  # All saints
        self.assertIn(date(2013, 12, 21), holidays)  # São Tomé Day
        self.assertIn(date(2013, 12, 25), holidays)  # XMas


class AngolaTest(GenericCalendarTest):
    cal_class = Angola

    def test_year_2018(self):
        holidays = self.cal.holidays_set(2018)
        # Dia de Ano Novo – 1 de Janeiro
        self.assertIn(date(2018, 1, 1), holidays)  # Ano Novo
        # Dia do Inicio da Luta Armada – 4 de Fevereiro
        self.assertIn(date(2018, 2, 4), holidays)
        # Dia do Carnaval – 13 de Fevereiro
        self.assertIn(date(2018, 2, 13), holidays)  # Entrudo
        # Dia Internacional da Mulher – 8 de Março
        self.assertIn(date(2018, 3, 8), holidays)
        # Dia da Paz – 4 de Abril
        self.assertIn(date(2018, 4, 4), holidays)  # Dia da Paz
        # Sexta Feira Santa – 30 de Março
        self.assertIn(date(2018, 3, 30), holidays)  # Sexta-Feira Santa
        # Páscoa – 01 de Abril
        self.assertIn(date(2018, 4, 1), holidays)  # Domingo de Páscoa
        # Dia Internacional do Trabalhador – 1 de Maio
        self.assertIn(date(2018, 5, 1), holidays)  # Dia do Trabalhador
        # Dia do Fundador da Nação e do Herói Nacional – 17 de Setembro
        self.assertIn(date(2018, 9, 17), holidays)
        # Dia dos Finados – 2 de Novembro
        # Dia da Independência Nacional – 11 de Novembro
        self.assertIn(date(2018, 11, 11), holidays)
        # Dia do Natal – 25 de Dezembro
        self.assertIn(date(2018, 12, 25), holidays)  # Natal

    def test_carnaval_label(self):
        holidays = self.cal.holidays(2018)
        holidays_dict = dict(holidays)
        label_carnaval = holidays_dict[date(2018, 2, 13)]
        self.assertEqual(label_carnaval, "Dia de Carnaval")

    def test_labour_day_label(self):
        holidays = self.cal.holidays(2020)
        holidays_dict = dict(holidays)
        label = holidays_dict[date(2020, 5, 1)]
        self.assertEqual(label, "Dia Internacional do Trabalhador")


class KenyaTest(GenericCalendarTest):
    cal_class = Kenya

    def test_year_2019(self):
        holidays = self.cal.holidays_set(2019)
        self.assertIn(date(2019, 1, 1), holidays)  # New Year
        self.assertIn(date(2019, 4, 19), holidays)  # Good Friday
        self.assertIn(date(2019, 4, 22), holidays)  # Easter Monday
        self.assertIn(date(2019, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2019, 6, 1), holidays)  # Madaraka Day
        self.assertIn(date(2019, 6, 5), holidays)  # Eid al-Fitr
        self.assertIn(date(2019, 8, 12), holidays)  # Eid al-Adha
        self.assertIn(date(2019, 10, 10), holidays)  # Moi Day (old name)
        self.assertIn(date(2019, 10, 20), holidays)  # Mashujaa Day
        self.assertIn(date(2019, 10, 21), holidays)  # Mashujaa Day shift
        self.assertIn(date(2019, 12, 12), holidays)  # Jamhuri Day
        self.assertIn(date(2019, 12, 25), holidays)  # Christmas Day
        self.assertIn(date(2019, 12, 26), holidays)  # Boxing Day (old name)
        self.assertIn(date(2019, 12, 31), holidays)  # New Years Eve
        self.assertEqual(len(holidays), 14)

    def test_year_2020(self):
        holidays = self.cal.holidays_set(2020)
        self.assertIn(date(2020, 1, 1), holidays)  # New Year
        self.assertIn(date(2020, 2, 11), holidays)  # Moi Memorial (as of 2020)
        self.assertIn(date(2020, 4, 10), holidays)  # Good Friday
        self.assertIn(date(2020, 4, 13), holidays)  # Easter Monday
        self.assertIn(date(2020, 5, 1), holidays)  # Labour Day
        self.assertIn(date(2020, 5, 24), holidays)  # Eid al-Fitr
        self.assertIn(date(2020, 6, 1), holidays)  # Madaraka Day
        self.assertIn(date(2020, 7, 31), holidays)  # Eid al-Adha
        self.assertIn(date(2020, 10, 10), holidays)  # Huduma Day
        self.assertIn(date(2020, 10, 20), holidays)  # Mashujaa Day
        self.assertIn(date(2020, 12, 12), holidays)  # Jamhuri Day
        self.assertIn(date(2020, 12, 25), holidays)  # Christmas Day
        self.assertIn(date(2020, 12, 26), holidays)  # Utamaduni Day
        self.assertIn(date(2020, 12, 31), holidays)  # New Years Eve
        self.assertEqual(len(holidays), 14)


class NigeriaTest(GenericCalendarTest):
    cal_class = Nigeria

    def test_year_2019(self):
        holidays = self.cal.holidays_set(2019)
        self.assertIn(date(2019, 1, 1), holidays)  # New Year
        self.assertIn(date(2019, 4, 19), holidays)  # Good Friday
        self.assertIn(date(2019, 4, 22), holidays)  # Easter Monday
        self.assertIn(date(2019, 5, 1), holidays)  # Workers Day
        self.assertIn(date(2019, 6, 5), holidays)  # Eid al-Fitr
        self.assertIn(date(2019, 6, 12), holidays)  # Democracy Day
        self.assertIn(date(2019, 10, 1), holidays)  # Independence Day
        self.assertIn(date(2019, 8, 12), holidays)  # Eid al-Adha
        self.assertIn(date(2019, 12, 25), holidays)  # Christmas Day
        self.assertIn(date(2019, 12, 26), holidays)  # Boxing Day

    def test_year_2000(self):
        holidays = self.cal.holidays_set(2000)
        self.assertIn(date(2000, 5, 29), holidays)  # Former Democracy Day

    def test_year_1998(self):
        holidays = self.cal.holidays_set(1998)
        self.assertNotIn(date(1998, 5, 29), holidays)  # No Democracy Day yet
        self.assertNotIn(date(1998, 6, 12), holidays)  # No Democracy Day yet



from datetime import date

from ..core import Calendar, MON, SAT, SUN
from ..astronomy import calculate_equinoxes
from ..registry_tools import iso_register


@iso_register('JP')
class Japan(Calendar):
    "Japan"

    # Japan uses the "western" workweek
    WEEKEND_DAYS = (SAT, SUN)

    FIXED_HOLIDAYS = Calendar.FIXED_HOLIDAYS + (
        (2, 11, "Foundation Day"),
        (4, 29, "Showa Day"),
        (5, 3, "Constitution Memorial Day"),
        (5, 4, "Greenery Day"),
        (5, 5, "Children's Day"),
        (11, 3, "Culture Day"),
        (11, 23, "Labour Thanksgiving Day"),
    )

    def get_fixed_holidays(self, year):
        """
        Fixed holidays for Japan.
        """
        days = super().get_fixed_holidays(year)
        if year >= 2016 and year != 2021:
            days.append((date(year, 8, 11), "Mountain Day"))
        # Change in Emperor
        if year < 2019:
            days.append((date(year, 12, 23), "The Emperor's Birthday"))
        if year > 2019:
            days.append((date(year, 2, 23), "The Emperor's Birthday"))
        # Lots of adjustments for new emperor
        if year == 2019:
            days.extend([
                (date(year, 4, 30), "Coronation Day"),
                (date(year, 5, 1), "Coronation Day"),
                (date(year, 5, 2), "Coronation Day"),
                (date(year, 5, 6), "Children's Day Observed"),
                (date(year, 8, 12), "Mountain Day Observed"),
                (date(year, 10, 22), "Enthronement Ceremony Day"),
                (date(year, 11, 4), "Culture Day Observed"),
            ])
        if year == 2020:
            days.extend([
                (date(year, 2, 24), "The Emperor's Birthday Observed"),
                (date(year, 5, 6), "Constitution Memorial Day Observed"),
                (date(year, 8, 10), "Mountain Day"),
            ])
            # Mountain Day is 8/10 this year for some reason
            # The next year that will be different is 2024
            days.remove((date(year, 8, 11), "Mountain Day"))
        if year == 2021:
            # Mountain Day is changed due to Tokyo's Olympics
            days.extend([
                (date(year, 8, 8), "Mountain Day"),
                (date(year, 8, 9), "Mountain Day Observed"),
            ])
        return days

    def get_variable_days(self, year):
        # usual variable days
        days = super().get_variable_days(year)
        equinoxes = calculate_equinoxes(year, 'Asia/Tokyo')
        coming_of_age_day = Japan.get_nth_weekday_in_month(year, 1, MON, 2)
        respect_for_the_aged = Japan.get_nth_weekday_in_month(year, 9, MON, 3)

        if year == 2020:
            # Health and Sports Day will continue on the 2nd monday
            # of October, except in 2020 when it will happen in July
            # https://www.timeanddate.com/holidays/japan/sports-day
            marine_day = date(2020, 7, 23)
            health_and_sport = date(2020, 7, 24)
        elif year == 2021:
            # Marine Day & Sport's Day are changed due to Tokyo's Olympics
            marine_day = date(year, 7, 22)
            health_and_sport = date(year, 7, 23)
        else:
            # Marine Day is on a Thursday in 2020 for some year
            # https://www.timeanddate.com/holidays/japan/sea-day
            marine_day = Japan.get_nth_weekday_in_month(year, 7, MON, 3)
            health_and_sport = Japan.get_nth_weekday_in_month(year, 10, MON, 2)

        # Health and Sports Day became "Sports Day" as of 2020
        if year < 2020:
            health_and_sport_label = "Health and Sports Day"
        else:
            health_and_sport_label = "Sports Day"

        days.extend([
            (coming_of_age_day, 'Coming of Age Day'),
            (marine_day, "Marine Day"),
            (equinoxes[0], "Vernal Equinox Day"),
            (respect_for_the_aged, "Respect-for-the-Aged Day"),
            (equinoxes[1], "Autumnal Equinox Day"),
            (health_and_sport, health_and_sport_label),
        ])

        return days


class JapanBank(Japan):
    """The Bank of Japan is closed additional days other than
    national holidays.
    https://www.boj.or.jp/en/about/outline/holi.htm/
    """

    FIXED_HOLIDAYS = Japan.FIXED_HOLIDAYS + (
        (1, 2, "New Year Bank Holiday"),
        (1, 3, "New Year Bank Holiday"),
        (12, 31, "New Year Bank Holiday"),
    )



from ..registry_tools import iso_register
from .core import UnitedStates


@iso_register('US-MN')
class Minnesota(UnitedStates):
    """Minnesota"""
    include_thanksgiving_friday = True
    include_columbus_day = False



from datetime import date, timedelta

from dateutil import relativedelta as rd

from ..core import MON, SUN, THU, TUE, WesternCalendar
from ..core import Holiday
from ..registry_tools import iso_register


@iso_register('US')
class UnitedStates(WesternCalendar):
    "United States of America"

    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        Holiday(
            date(2000, 7, 4),
            'Independence Day',
            indication='July 4',
        ),
    )

    @property
    def observance_shift(self):
        return Holiday.nearest_weekday

    # Veterans day label
    include_veterans_day = False
    veterans_day_label = 'Veterans Day'

    # MLK
    martin_luther_king_label = 'Birthday of Martin Luther King, Jr.'

    include_thanksgiving_friday = False
    thanksgiving_friday_label = "Thanksgiving Friday"
    # Some states don't include Washington's Birthday, or move it to December.
    include_federal_presidents_day = True
    presidents_day_label = "Washington's Birthday"

    include_lincoln_birthday = False

    # Columbus day is included by default
    include_columbus_day = True
    columbus_day_label = "Columbus Day"
    # Confederation day
    include_confederation_day = False
    # Jefferson Davis Birthday.
    include_jefferson_davis_birthday = False

    # Include Cesar Chavez day(s)
    include_cesar_chavez_day = False
    # Patriot's day
    include_patriots_day = False

    # Boxing day label is not "boxing day" in the US
    boxing_day_label = "Day After Christmas"

    # Election Day
    # To include every year?
    include_election_day_every_year = False
    # To include on even years?
    include_election_day_even = False
    # NOTE: if it's included on every year, it'll also be included
    # on even years. Setting these two flags to ON will give priority to the
    # yearly flag.
    election_day_label = "Election Day"
    # Inauguration Day
    include_inauguration_day = False

    # National Memorial Day
    national_memorial_day_label = "Memorial Day"

    # Some regional variants
    include_mardi_gras = False
    include_fat_tuesday = False
    fat_tuesday_label = "Mardi Gras"

    # Juneteenth
    include_juneteenth = False

    # Shift day mechanism
    # These days won't be shifted to next MON or previous FRI
    shift_exceptions = (
        # Example:
        # (11, 11),  # Veterans day won't be shifted
    )

    @staticmethod
    def is_presidential_year(year):
        return (year % 4) == 0

    def get_election_date(self, year):
        """
        Return the Election Day *Date*

        Definition: on an election year, "the Tuesday next after the first
        Monday in the month of November".
        """
        first_monday_november = self.get_nth_weekday_in_month(year, 11, MON)
        return self.get_nth_weekday_in_month(
            year, 11, TUE, start=first_monday_november
        )

    def get_election_day(self, year):
        """
        Return the Election Day
        """
        return self.get_election_date(year), self.election_day_label

    def get_thanksgiving_friday(self, year):
        """
        Thanksgiving friday is on the day following Thanksgiving Day
        """
        thanksgiving = UnitedStates.get_nth_weekday_in_month(year, 11, THU, 4)
        thanksgiving_friday = thanksgiving + timedelta(days=1)
        return thanksgiving_friday, self.thanksgiving_friday_label

    def get_confederate_day(self, year):
        """
        Confederate memorial day is on the 4th MON of April.
        """
        day = self.get_nth_weekday_in_month(year, 4, MON, 4)
        return day, "Confederate Memorial Day"

    def get_jefferson_davis_birthday(self, year):
        """
        The first MON of June is Jefferson Davis Birthday
        """
        return (
            self.get_nth_weekday_in_month(year, 6, MON, 1),
            "Jefferson Davis Birthday"
        )

    def get_martin_luther_king_date(self, year):
        if year < 1985:
            raise ValueError(
                "Martin Luther King Day became a holiday in 1985"
            )
        return date(year, 1, 1) + rd.relativedelta(weekday=rd.MO(3))

    def get_martin_luther_king_day(self, year):
        """
        Return holiday record for Martin Luther King Jr. Day.
        """
        return Holiday(
            self.get_martin_luther_king_date(year),
            self.martin_luther_king_label,
            indication="3rd Monday in January",
        )

    def get_presidents_day(self, year):
        """
        Presidents Day is on the 3rd MON of February

        May be called Washington's or Lincoln's birthday
        """
        return Holiday(
            date(year, 2, 1) + rd.relativedelta(weekday=rd.MO(3)),
            self.presidents_day_label,
            indication="3rd Monday in February",
        )

    def get_cesar_chavez_days(self, year):
        """
        Cesar Chavez day is on 31st of March

        Will return a list of days, because in some states (California),
        it can float to MON if it happens on SUN.
        """
        days = [(date(year, 3, 31), "Cesar Chavez Day")]
        return days

    def get_patriots_day(self, year):
        """3rd Monday of April"""
        return self.get_nth_weekday_in_month(year, 4, MON, 3), "Patriots Day"

    def get_columbus_day(self, year):
        """
        Columbus day is on the 2nd MON of October.

        Only half of the states recognize it.
        """
        return Holiday(
            date(year, 10, 1) + rd.relativedelta(weekday=rd.MO(2)),
            self.columbus_day_label,
            indication="2nd Monday in October",
        )

    def get_lincoln_birthday(self, year):
        """
        February the 2nd is Lincoln's birthday in the following States:

        * Connecticut,
        * Illinois,
        * Missouri,
        * New York
        """
        return date(year, 2, 12), "Lincoln's Birthday"

    def get_inauguration_date(self, year):
        """
        If the year is an Inauguration Year, will return the Inauguration Day
        date.

        If this day falls on SUN, it's replaced by the next MON.
        If the year is not a Inauguration Year, it raises a ValueError.
        """
        if ((year - 1) % 4) != 0:
            raise ValueError(
                f"The year {year} is not an Inauguration Year")
        inauguration_day = date(year, 1, 20)
        if inauguration_day.weekday() == SUN:
            inauguration_day = date(year, 1, 21)
        return inauguration_day

    def get_national_memorial_day(self, year):
        return Holiday(
            date(year, 5, 31) + rd.relativedelta(weekday=rd.MO(-1)),
            self.national_memorial_day_label,
            indication="Last Monday in May",
        )

    def get_juneteenth_day(self, year):
        """
        Return Juneteenth Day
        """
        # Juneteenth started to be a federal holiday in 2021
        if year < 2021:
            raise ValueError("Juneteenth became a federal holiday in 2021")
        return Holiday(
            date(year, 6, 19),
            "Juneteenth National Independence Day",
            indication="June 19th",
        )

    def get_variable_days(self, year):  # noqa: C901
        # usual variable days
        days = super().get_variable_days(year)

        days += [
            self.get_veterans_day(year),
            self.get_national_memorial_day(year),
            Holiday(
                date(year, 9, 1) + rd.relativedelta(weekday=rd.MO(1)),
                "Labor Day",
                indication="1st Monday in September",
            ),

            Holiday(
                date(year, 11, 1) + rd.relativedelta(weekday=rd.TH(4)),
                "Thanksgiving Day",
                indication="4th Thursday in November",
            ),
        ]

        # Martin Luther King's Day started only in 1985
        if year >= 1985:
            days.append(self.get_martin_luther_king_day(year))

        if self.include_mardi_gras:
            days.append(self.get_mardi_gras(year))

        if self.include_federal_presidents_day:
            days.append(self.get_presidents_day(year))

        if self.include_lincoln_birthday:
            days.append(self.get_lincoln_birthday(year))

        if self.include_cesar_chavez_day:
            days.extend(self.get_cesar_chavez_days(year))

        if self.include_patriots_day:
            days.append(self.get_patriots_day(year))

        if self.include_columbus_day:
            days.append(self.get_columbus_day(year))

        if self.include_confederation_day:
            days.append(self.get_confederate_day(year))

        if self.include_jefferson_davis_birthday:
            days.append(self.get_jefferson_davis_birthday(year))

        ind = "January 20 (or 21st if Sunday) following an election year"
        if self.include_inauguration_day:
            # Is it a "Inauguration year"?
            if UnitedStates.is_presidential_year(year - 1):
                days.append(
                    Holiday(
                        self.get_inauguration_date(year),
                        "Inauguration Day",
                        indication=ind,
                    ),
                )

        if self.include_election_day_every_year:
            days.append(self.get_election_day(year))
        elif self.include_election_day_even:
            if (year % 2) == 0:
                days.append(self.get_election_day(year))

        if self.include_thanksgiving_friday:
            days.append(
                self.get_thanksgiving_friday(year)
            )

        if self.include_juneteenth and year >= 2021:
            days.append(self.get_juneteenth_day(year))

        return days

    def get_veterans_day(self, year):
        """
        Return Veterans Day (November 11th).

        Placed here because some States are renaming it.
        """
        return Holiday(
            date(year, 11, 11),
            self.veterans_day_label,
            indication='Nov 11',
        )

    def get_fixed_holidays(self, year):
        days = super().get_fixed_holidays(year)
        if self.include_veterans_day:
            days.append(self.get_veterans_day(year))
        return days


class FederalReserveSystem(UnitedStates):
    "Board of Governors of the Federal Reserve System of the USA"

    include_juneteenth = True



from datetime import date

from ..core import WesternCalendar, MON, FRI
from ..exceptions import CalendarError
from ..registry_tools import iso_register


@iso_register('ZA')
class SouthAfrica(WesternCalendar):
    "South Africa"
    include_good_friday = True
    include_christmas = True

    def holidays(self, year=None):
        if year < 1910:
            raise CalendarError("It's not possible to compute holidays prior"
                                " to 1910 for South Africa.")
        return super().holidays(year)

    def get_easter_monday_or_family_day(self, year):
        if year < 1980:
            label = "Easter Monday"
        else:
            label = "Family Day"
        return self.get_easter_monday(year), label

    def get_fixed_holidays(self, year):  # noqa: C901
        days = super().get_fixed_holidays(year)
        if year >= 1990:
            days.append((date(year, 3, 21), 'Human Rights Day'))

        # Van Riebeeck's day & Founder's day
        if year >= 1952 and year <= 1973:
            days.append((date(year, 4, 6), "Van Riebeeck's Day"))
        if year >= 1980 and year <= 1994:
            days.append((date(year, 4, 6), "Founder's Day"))

        if year >= 1994:
            days.append((date(year, 4, 27), "Freedom Day"))

        # Workers day established in 1995 to May 1st
        if year >= 1995:
            days.append((date(year, 5, 1), "Workers' Day"))

        if year <= 1951:
            days.append((date(year, 5, 24), "Victoria Day / Empire Day"))

        # May 31st: Union Day & Republic Day
        if year <= 1960:
            days.append((date(year, 5, 31), "Union Day"))
        elif year <= 1993:
            days.append((date(year, 5, 31), "Republic Day"))

        if year >= 1995:
            days.append((date(year, 6, 16), "Youth Day"))

        if year > 1960 and year <= 1973:
            days.append((date(year, 7, 10), "Family Day"))

        if year >= 1995:
            days.append((date(year, 8, 9), "National Women's Day"))
            days.append((date(year, 9, 24), "Heritage Day"))

        if year >= 1952 and year <= 1993:
            days.append((date(year, 10, 10), "Kruger Day"))

        if year <= 1951:
            december_16th_label = "Dingaan's Day"
        elif 1952 <= year <= 1979:
            december_16th_label = "Day of the Covenant"
        elif 1980 <= year <= 1994:
            december_16th_label = "Day of the Vow"
        else:
            december_16th_label = "Day of Reconciliation"
        days.append((date(year, 12, 16), december_16th_label))

        # Boxing day renamed
        boxing_day_label = "Boxing Day"
        if year >= 1980:
            boxing_day_label = "Day of Goodwill"
        days.append((date(year, 12, 26), boxing_day_label))

        return days

    def get_variable_days(self, year):
        days = super().get_variable_days(year)

        days.append(self.get_easter_monday_or_family_day(year))

        # Workers day was first friday of may 1987-1989
        if 1987 <= year <= 1989:
            days.append(
                (self.get_nth_weekday_in_month(year, 5, FRI), "Workers' Day")
            )

        if year <= 1993:
            days.append((self.get_ascension_thursday(year), "Ascension Day"))

        # Queen's Birthday on the 2nd Monday of july 1952-1960
        if 1952 <= year <= 1960:
            days.append((
                self.get_nth_weekday_in_month(year, 7, MON, 2),
                "Queen's Birthday"
            ))

        # King's Birthday on the first Monday of August 1910-1951
        if 1910 <= year <= 1951:
            days.append((
                self.get_nth_weekday_in_month(year, 8, MON),
                "King's Birthday"
            ))

        if year >= 1952 and year <= 1979:
            days.append((self.get_nth_weekday_in_month(year, 9, MON),
                         "Settlers' Day"))
        return days

    def get_calendar_holidays(self, year):  # noqa: C901
        days = super().get_calendar_holidays(year)

        # Other one-offs. Don't shift these
        if year == 1999:
            days.append((date(year, 6, 2), "National Elections"))
            days.append((date(year, 12, 31), "Y2K"))
        if year == 2000:
            # 2 January 2000 public holidays to accommodate the Y2K changeover,
            # 3 January 2000 because the previous holiday was a Sunday
            days.append((date(year, 1, 2), "Y2K"))
            days.append((date(year, 1, 3), "Y2K"))
        if year == 2001:
            days.append((date(year, 1, 2), "Y2K"))
        if year == 2004:
            days.append((date(year, 4, 14), "National Elections"))
        if year == 2006:
            days.append((date(year, 3, 1), "Local Elections"))
        if year == 2008:
            # 2 May 2008 was declared a public holiday when Human Rights Day
            # and Good Friday coincided on 21 March 2008
            days.append((date(year, 5, 2), "Special Human Rights"))
        if year == 2009:
            days.append((date(year, 4, 22), "National Elections"))
        if year == 2011:
            days.append((date(year, 5, 18), "Local Elections"))
            days.append((date(year, 12, 27), "Special Day of Goodwill"))
        if year == 2014:
            days.append((date(year, 5, 7), "National Elections"))
        if year == 2016:
            days.append((date(year, 8, 3), "Local Elections"))
        if year == 2019:
            days.append((date(year, 5, 8), "National Elections"))

        return days



from ..core import (
    WesternMixin, IslamicMixin, ChineseNewYearCalendar,
    SAT, SUN
)
from ..registry_tools import iso_register


@iso_register('PH')
class Philippines(WesternMixin, IslamicMixin, ChineseNewYearCalendar):
    "Philippines"
    # Civil holidays
    include_labour_day = True
    include_new_years_eve = True

    # Christian holiday
    include_holy_thursday = True
    holy_thursday_label = "Maundy Thursday"
    include_good_friday = True
    include_easter_saturday = True
    include_easter_sunday = True
    easter_saturday_label = "Black Saturday"
    include_all_saints = True
    include_all_souls = True
    include_immaculate_conception = True
    include_christmas_eve = True

    # Islamic holidays
    include_eid_al_fitr = True
    eid_al_fitr_label = "Eid'l Fitr"
    include_eid_al_adha = True
    day_of_sacrifice_label = "Eid'l Adha"

    WEEKEND_DAYS = (SAT, SUN)

    FIXED_HOLIDAYS = ChineseNewYearCalendar.FIXED_HOLIDAYS + (
        (2, 25, "EDSA Revolution Anniversary"),
        (4, 9, "Araw ng Kagitingan"),
        (6, 12, "Independence Day"),
        (8, 21, "Ninoy Aquino Day"),
        (8, 30, "National Heroes' Day"),
        (11, 30, "Bonifacio Day"),
        (12, 30, "Rizal Day"),
    )



from datetime import date
from ..core import WesternCalendar
from ..registry_tools import iso_register


@iso_register('SI')
class Slovenia(WesternCalendar):
    'Slovenia'

    # Christian holidays
    include_easter_sunday = True
    include_easter_monday = True
    include_whit_sunday = True
    include_assumption = True
    include_christmas = True

    # Civil holidays
    include_labour_day = True
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (2, 8, "Preseren Day, the Slovenian Cultural Holiday"),
        (4, 27, "Day of Uprising Against Occupation"),
        (5, 2, "Labour Day"),  # In Slovenia, Labour Day is 2-day long.
        (6, 25, "Statehood Day"),
        (10, 31, "Reformation Day"),
        (11, 1, "Day of Remembrance of the Dead"),
        (12, 26, "Independence and Unity Day"),
    )

    def get_variable_days(self, year):
        days = super().get_variable_days(year)

        # From 1955 until May 2012, when the National Assembly of Slovenia
        # passed the Public Finance Balance Act, 2 January was a work-free day.
        # It has been re-introduced in 2017.
        # Source - Wikipedia
        # https://en.wikipedia.org/wiki/Public_holidays_in_Slovenia
        if 1955 <= year <= 2012 or year >= 2017:
            days.append((date(year, 1, 2), "January 2nd"))

        return days



from datetime import date

from ..core import WesternCalendar, MON, TUE, WED, THU, FRI, SAT
from ..astronomy import solar_term
from ..registry_tools import iso_register


# https://www.feriados.cl
# http://www.feriadoschilenos.cl

@iso_register('CL')
class Chile(WesternCalendar):
    "Chile"
    FIXED_HOLIDAYS = WesternCalendar.FIXED_HOLIDAYS + (
        (5, 21, "Navy Day"),
        (9, 18, "National holiday"),
        (9, 19, "Army holiday"),
        (12, 31, "Banking Holiday"),
    )
    # Civil holidays
    # Labor day (Law 2.200 and Law 18.018)
    include_labour_day = True

    # Christian holidays
    # Holy Week (Law 2.977)
    include_good_friday = True
    include_easter_saturday = True
    # Assumption (Law 2.977)
    include_assumption = True
    # All Saints (Law 2.977)
    include_all_saints = True
    # Immaculate Conception (Law 2.977)
    include_immaculate_conception = True

    def get_variable_days(self, year):  # noqa: C901
        days = super().get_variable_days(year)

        # Indigenous Peoples (Law 21.357)
        indigenous_peoples_day = date(year, 6, 21)
        if year == 2021:
            days.append((indigenous_peoples_day, 'Indigenous Peoples Day'))
        elif year > 2021:
            june_solstice = solar_term(year, 90, 'America/Santiago')
            days.append((june_solstice, 'Indigenous Peoples Day'))

        # Saint Peter and Saint Paul (Law 18.432)
        peter_paul = date(year, 6, 29)
        if year < 2000:
            days.append((peter_paul, 'Saint Peter and Saint Paul'))
        else:
            # floating monday (Law 19.668)
            if peter_paul.weekday() in [TUE, WED, THU]:
                days.append((Chile.get_nth_weekday_in_month(year, 6, MON, 4),
                            'Saint Peter and Saint Paul'))
            elif peter_paul.weekday() == FRI:
                days.append((date(year, 7, 2), 'Saint Peter and Saint Paul'))
            else:
                days.append((peter_paul, 'Columbus Day'))

        # Our Lady of Mount Carmel (Law 20.148)
        if year >= 2007:
            days.append((date(year, 7, 16), "Our Lady of Mount Carmel"))

        # National Bridge Days (Law 20.215)
        if year >= 2007:
            september_17 = date(year, 9, 17)
            if september_17.weekday() == MON:
                days.append((september_17, '"Bridge" holiday'))
            september_20 = date(year, 9, 20)
            if september_20.weekday() == FRI:
                days.append((september_20, '"Bridge" holiday'))

        # Additional Friday (Law 20.983)
        if year >= 2017:
            september_18 = date(year, 9, 18)
            if (september_18.weekday() == SAT):
                days.append((september_17, '"Bridge" holiday'))

        # Meeting of the two Worlds (Law 3.810)
        columbus_day = date(year, 10, 12)
        if year < 2000:
            days.append((columbus_day, 'Columbus Day'))
        else:
            # floating monday (Law 19.668)
            if columbus_day.weekday() in [TUE, WED, THU]:
                days.append((Chile.get_nth_weekday_in_month(year, 10, MON, 2),
                            'Columbus Day'))
            elif columbus_day.weekday() == FRI:
                days.append((date(year, 10, 15), 'Columbus Day'))
            else:
                days.append((columbus_day, 'Columbus Day'))

        # Reformation Day (Law 20.299)
        if year >= 2008:
            reformation_day = date(year, 10, 31)
            if reformation_day.weekday() == WED:
                reformation_day = date(year, 11, 2)
            elif reformation_day.weekday() == TUE:
                reformation_day = date(year, 10, 27)

            days.append((reformation_day, "Reformation Day"))

        return days
